﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;
using System.Reflection;
using LiveMesh.MeshService;

namespace LiveMesh
{
    public class FontListBox : OwnerDrawnListBox
    {
        const int FONT_SIZE = 10;
        const int DRAW_OFFSET = 0;
        Bitmap backBuffer;

        public FontListBox()
        {

            // Determine what the item height should be
            // by adding 30% padding after measuring
            // the letter A with the selected font.
            Graphics g = this.CreateGraphics();
            this.ItemHeight = (int)(g.MeasureString("人", new Font("Tahoma", 12, FontStyle.Regular)).Height * 2);
            g.Dispose();
        }

        // Return the name of the selected font.
        public MeshPicInfo SelectedFaceName
        {
            get
            {
                return ((MeshPicInfo)this.Items[this.SelectedIndex]);
            }
        }

        // Determine what the text color should be
        // for the selected item drawn as highlighted
        Color CalcTextColor(Color backgroundColor)
        {
            if (backgroundColor.Equals(Color.Empty))
                return Color.Black;

            int sum = backgroundColor.R + backgroundColor.G + backgroundColor.B;

            if (sum > 256)
                return Color.Black;
            else
                return Color.White;
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            Font font;
            Color fontColor;

            // The base class contains a bitmap, offScreen, for constructing
            // the control and is rendered when all items are populated.
            // This technique prevents flicker.
            Graphics gOffScreen = Graphics.FromImage(this.OffScreen);
            gOffScreen.Clear(this.BackColor);

            string path = System.IO.Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetName().CodeBase);

            //修改系统背景颜色为默认颜色
            this.BackColor = SysColor;
            //gOffScreen.FillRectangle(new SolidBrush(SystemColors.ActiveCaption), this.ClientRectangle);
            AlphaImage bmp = AlphaImage.CreateFromFile(path + @"\Resources\bg.png");
            Rectangle imgRect = new Rectangle(0, 0, this.Width, this.Height);
            bmp.Draw(gOffScreen, imgRect);

            int itemTop = 0;
            AlphaImage bmpIcon;
            Rectangle imgIconRect;
            // Draw the fonts in the list.
            for (int n = this.VScrollBar.Value; n < this.VScrollBar.Value + DrawCount; n++)
            {
                // If the font name contains "dings" it needs to be displayed
                // in the list box with a readable font with the default font.
                font = new Font("Tahoma", 10, FontStyle.Regular);

                // Draw the selected item to appear highlighted
                if (n == this.SelectedIndex)
                {
                    gOffScreen.FillRectangle(new SolidBrush(SystemColors.ScrollBar),
                        1,
                        itemTop + 1,
                        // If the scroll bar is visible, subtract the scrollbar width
                        // otherwise subtract 2 for the width of the rectangle
                        this.ClientSize.Width - (this.VScrollBar.Visible ? this.VScrollBar.Width : 2),
                        this.ItemHeight);
                    fontColor = CalcTextColor(SystemColors.Highlight);
                }
                else
                    fontColor = Color.White;

                // Draw the item

                //ICON缩略图 只针对下载管理应用
                if (ParentType != null)
                {
                    switch (ParentType)
                    {
                        case "Mesh":
                            ParentType = "icon_06";
                            break;
                        //case "POLO衫":
                        //    ParentType = "t";
                        //    break;
                        //case "裤子":
                        //    ParentType = "k";
                        //    break;
                        default:
                            break;
                    }
                    bmp = AlphaImage.CreateFromFile(path + @"\Resources\" + ParentType + ".png");
                    imgIconRect = new Rectangle(DRAW_OFFSET + 3, itemTop + 2, 36, 35);
                    bmp.Draw(gOffScreen, imgIconRect);
                    gOffScreen.DrawString(((MeshPicInfo)this.Items[n]).Title, font, new SolidBrush(fontColor), DRAW_OFFSET + 55, itemTop + 12);
                    //gOffScreen.DrawString(((InfoClass)this.Items[n]).Author, font, new SolidBrush(fontColor), this.Width - 70, itemTop + 7);
                }
                else
                {
                    gOffScreen.DrawString(((MeshPicInfo)this.Items[n]).Title, font, new SolidBrush(fontColor), DRAW_OFFSET + 5, itemTop + 7);
                    //gOffScreen.DrawString(((InfoClass)this.Items[n]).Author, font, new SolidBrush(fontColor), this.Width - 70, itemTop + 7);
                }
                gOffScreen.DrawLine(new Pen(SystemColors.Highlight), DRAW_OFFSET, itemTop, this.Width, itemTop);
                itemTop += this.ItemHeight;
            }
            // Draw the list box
            e.Graphics.DrawImage(this.OffScreen, 1, 1);

            gOffScreen.Dispose();
        }


        // Draws the external border around the control.

        protected override void OnPaintBackground(PaintEventArgs e)
        {
            backBuffer = new Bitmap(this.Width, this.Height, PixelFormat.Format32bppRgb);

            e.Graphics.DrawRectangle(new Pen(Color.Black), 0, 0, this.ClientSize.Width - 1, this.ClientSize.Height - 1);
        }


        public void SelectedChanged(bool isSelected)
        {
            if (isSelected)
            {
                Height = 69;
            }
        }

    }
}
