﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using LiveMesh.MeshService;

namespace LiveMesh
{
    public delegate void SelItemInfo(MeshPicInfo info,bool othSel);
    public class OwnerDrawnListBox : Control
    {
        const int SCROLL_WIDTH = 12;
        int itemHeight = -1;
        int selectedIndex = -1;
        bool otherSel = false;
        Bitmap offScreen;
        VScrollBar vs;
        ArrayList items;
        public event SelItemInfo Selected;
        public event SelItemInfo MovedOn;
        public OwnerDrawnListBox()
        {
            this.vs = new VScrollBar();
            this.vs.Parent = this;
            this.vs.Visible = false;
            this.vs.SmallChange = 1;
            this.vs.ValueChanged += new EventHandler(this.ScrollValueChanged);

            this.items = new ArrayList();
        }

        public ArrayList Items
        {
            get
            {
                return this.items;
            }
        }

        protected Bitmap OffScreen
        {
            get
            {
                return this.offScreen;
            }
        }

        protected VScrollBar VScrollBar
        {
            get
            {
                return this.vs;
            }
        }
        //接受父窗体背景配色
        public Color SysColor
        {
            get;
            set;
        }
        //父窗体可见图标数
        public int ViewCount
        {
            get;
            set;
        }
        //接受父窗体传递类型(音乐 主题等)
        public string ParentType
        {
            get;
            set;
        }
        public event EventHandler SelectedIndexChanged;

        // Raise the SelectedIndexChanged event
        protected virtual void OnSelectedIndexChanged(EventArgs e)
        {
            if (this.SelectedIndexChanged != null)
                this.SelectedIndexChanged(this, e);
        }

        protected override void OnMouseDown(MouseEventArgs e)
        {
            this.SelectedIndex = this.vs.Value + (e.Y / this.ItemHeight);

            // Invalidate the control so we can draw the item as selected.
            this.Refresh();
            if (e.Button == MouseButtons.Left)
            {
                if (this.selectedIndex >= this.items.Count)
                {
                    this.selectedIndex = 0;
                    this.Refresh();
                }
                else
                    Selected((MeshPicInfo)Items[this.SelectedIndex], false);
                    //OnKeyDown(new KeyEventArgs(Keys.Enter));
            }
            base.OnMouseDown(e);
        }

        // Get or set index of selected item.
        public int SelectedIndex
        {
            get
            {
                return this.selectedIndex;
            }

            set
            {
                this.selectedIndex = value;

                if (this.SelectedIndexChanged != null)
                    this.SelectedIndexChanged(this, EventArgs.Empty);

            }
        }

        protected void ScrollValueChanged(object o, EventArgs e)
        {
            this.Refresh();
        }

        protected virtual int ItemHeight
        {
            get
            {
                return this.itemHeight;
            }

            set
            {
                this.itemHeight = value;
            }
        }

        // If the requested index is before the first visible index then set the
        // first item to be the requested index. If it is after the last visible
        // index, then set the last visible index to be the requested index.
        public void EnsureVisible(int index)
        {
            if (index < this.vs.Value)
            {
                this.vs.Value = index;
                this.Refresh();
            }
            else if (index >= this.vs.Value + this.DrawCount)
            {
                this.vs.Value = index - this.DrawCount + 1;
                this.Refresh();
            }
        }


        // Need to set focus to the control when it
        // is clicked so that keyboard events occur.
        protected override void OnClick(EventArgs e)
        {
            //this.Focus();
            base.OnClick(e);
        }

        // Selected item moves when you use the keyboard up/down keys.
        protected override void OnKeyDown(KeyEventArgs e)
        {
            //if (this.selectedIndex < 0)
            //{
            //    return;
            //}
            if (this.selectedIndex >= this.Items.Count)
            {
                return;
            }
            switch (e.KeyCode)
            {
                case Keys.Down:
                    if (this.SelectedIndex < this.vs.Maximum)
                    {
                        EnsureVisible(++this.SelectedIndex);
                        this.Refresh();
                    }
                    otherSel = false;
                    if (this.selectedIndex < 0)
                    {
                        return;
                    }
                    MovedOn((MeshPicInfo)Items[this.SelectedIndex], otherSel);
                    break;
                case Keys.Up:
                    if (this.selectedIndex == 0)
                    {
                        otherSel = true;
                    }
                    if (this.SelectedIndex > this.vs.Minimum)
                    {
                        otherSel = false;
                        EnsureVisible(--this.SelectedIndex);
                        this.Refresh();
                    }
                    if (this.selectedIndex < 0)
                    {
                        return;
                    }
                    MovedOn((MeshPicInfo)Items[this.SelectedIndex], otherSel);
                    break;
                case Keys.Enter:
                    if (this.selectedIndex < 0 || this.SelectedIndex > this.items.Count)
                    {
                        return;
                    }
                    else
                        Selected((MeshPicInfo)Items[this.SelectedIndex], otherSel);
                    //(((Control)Items[this.SelectedIndex]) as IExtendedFontListbox).SelectedChanged(true);
                    break;
                case Keys.PageDown:
                    this.SelectedIndex = Math.Min(this.vs.Maximum, this.SelectedIndex + this.DrawCount);
                    EnsureVisible(this.SelectedIndex);
                    this.Refresh();
                    break;
                case Keys.PageUp:
                    this.SelectedIndex = Math.Max(this.vs.Minimum, this.SelectedIndex - this.DrawCount);
                    EnsureVisible(this.SelectedIndex);
                    this.Refresh();
                    break;
                case Keys.Home:
                    this.SelectedIndex = 0;
                    EnsureVisible(this.SelectedIndex);
                    this.Refresh();
                    break;
                case Keys.End:
                    this.SelectedIndex = this.items.Count - 1;
                    EnsureVisible(this.SelectedIndex);
                    this.Refresh();
                    break;
            }

            base.OnKeyDown(e);
        }

        // Calculate how many items we can draw given the height of the control.
        protected int DrawCount
        {
            get
            {
                if (this.vs.Value + this.vs.LargeChange > this.vs.Maximum)
                    return this.vs.Maximum - this.vs.Value + 1;
                else
                    return this.vs.LargeChange;
            }
        }

        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);
            if (offScreen != null)
                offScreen.Dispose();

            offScreen = new Bitmap(this.Width, this.Height);


            int viewableItemCount = this.ClientSize.Height / this.ItemHeight;

            this.vs.Bounds = new Rectangle(this.ClientSize.Width - SCROLL_WIDTH,
                0,
                SCROLL_WIDTH,
                this.ClientSize.Height);


            // Determine if scrollbars are needed
            if (this.items.Count > viewableItemCount)
            {
                this.vs.Visible = true;
                this.vs.LargeChange = viewableItemCount;
                this.offScreen = new Bitmap(this.ClientSize.Width - SCROLL_WIDTH - 1, this.ClientSize.Height - 2);
            }
            else
            {
                this.vs.Visible = false;
                this.vs.LargeChange = this.items.Count;
                this.offScreen = new Bitmap(this.ClientSize.Width - 1, this.ClientSize.Height - 2);
            }

            this.vs.Maximum = this.items.Count - 1;
        }
    }
}
